﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Windows.Forms;

namespace ADCLtoADCH
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        /* some not very well defined variables :)
        adclFiles: will keep the .adcl files list
        adclCount: files count
        adclDirectory: path for files, hardcoded to {AltDVB mail folder}/Channels
        */
        IEnumerable<string> adclFiles;
        int adclCount = 0;
        string adclDirectory = Application.StartupPath + "\\Channels";

        private void Form1_Load(object sender, EventArgs e)
        {
            // URL of AltDVB website, to be launched from the main window
            aboutLink.Links.Add(0, aboutLink.Text.Length, "https://www.altx.ro/projects/altdvb/");

            // enumerating files, if directory found, else - do nothing
            if (Directory.Exists(adclDirectory))
            {
                adclFiles = Directory.EnumerateFiles(adclDirectory, "*.adcl", SearchOption.TopDirectoryOnly);
                adclCount = adclFiles.Count();
            }

            // if any files were found - preparation to enable UI to make a conversion
            if (adclCount != 0)
            {
                startConversionButton.Enabled = true;
                conversionLabel.Text = "Converter has detected " + adclCount.ToString() + " files to be processed.";
                conversionLabel.ForeColor = Color.Green;
                conversionProgress.Maximum = adclCount;
                conversionProgress.Step = 1;
                conversionProgress.Value = 0;
            }
        }

        private void startConversionButton_Click(object sender, EventArgs e)
        {
            int filesCount = 0;

            // creating directory for moving old channel list files after a successfull conversion
            if(!Directory.Exists(Application.StartupPath + "\\old_Channels"))
            {
                try
                {
                    Directory.CreateDirectory(Application.StartupPath + "\\old_Channels");
                }
                catch
                {
                    MessageBox.Show("An error has occured: no rights to write to the current directory. Please fix this issue, you can try to copy this application and Channels directory to the other path and try again.", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }

            // launch conversion for each file, with progress indication after each step
            foreach (string adclFile in adclFiles)
            {
                try
                {
                    fileConverter(adclFile);
                }
                catch (Exception err)
                {
                    MessageBox.Show("An error has occured, when attempt was made to convert the file: " + Path.GetFileName(adclFile) + ". Please try to check this file, eventually remove it to re-try conversion for the rest of the files. Error message: " + err.Message, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    conversionProgressLabel.Text = "Progress: an error occured, when attempt was made to convert the file: " + Path.GetFileName(adclFile);
                    return;
                }
                finally
                {
                    filesCount++;
                    conversionProgress.Value++;
                    conversionProgressLabel.Text = "Progress: " + filesCount.ToString() + " file (" + Path.GetFileName(adclFile) + ") processed.";
                }
            }
            // expected to happen only if the whole process was successfull
            startConversionButton.Enabled = false;
            MessageBox.Show("Conversion was successfull. If AltDVB binaries are already updated, please try to run AltDVB and check if channel lists are available and complete.",this.Text,MessageBoxButtons.OK,MessageBoxIcon.Information);
            conversionProgressLabel.Text = "Progress: operation was completed successfully.";
        }

        private void fileConverter(string adclFile)
        {
            // copy old channel list file to a backup directory
            if ((Directory.Exists(Application.StartupPath + "\\old_Channels")) && (File.Exists(adclFile)))
            {
                try
                {
                    File.Copy(adclFile, Application.StartupPath + "\\old_Channels\\" + Path.GetFileName(adclFile), true);
                }
                catch
                {
                    MessageBox.Show("An error has occured: no rights to write to the current directory. Please fix this issue, you can try to copy this application and Channels directory to the other path and try again.", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }

            /* file conversion, idea behind this is pretty straightforward:
               - read the file
               - use 511 bytes chunks (a size of one record in .adcl file)
               - transpose layout to new 1024 bytes record size
               - write upgraded chunk to a new file, with the same name and corrected extension
            */
            using (Stream source = File.OpenRead(adclFile))
            {
                byte[] buffer = new byte[511];
                byte[] buffer2 = new byte[1024];
                var chunkCount = source.Length / 511;
                FileStream incomingFile = new FileStream((adclFile.Substring(0, adclFile.Length - 1)) + "h", FileMode.OpenOrCreate, FileAccess.ReadWrite, FileShare.ReadWrite);

                for (int i = 0; i < chunkCount + 1; i++)
                {
                    source.Position = i * 511;
                    int size = source.Read(buffer, 0, 511);

                    Buffer.BlockCopy(buffer, 0, buffer2, 0, 32);
                    Buffer.BlockCopy(buffer, 32, buffer2, 44, 32);
                    Buffer.BlockCopy(buffer, 64, buffer2, 172, 32);
                    Buffer.BlockCopy(buffer, 96, buffer2, 300, 6);
                    Buffer.BlockCopy(buffer, 102, buffer2, 307, 197);
                    Buffer.BlockCopy(buffer, 299, buffer2, 664, 160);
                    Buffer.BlockCopy(buffer, 466, buffer2, 984, 2);
                    Buffer.BlockCopy(buffer, 469, buffer2, 986, 1);
                    Buffer.BlockCopy(buffer, 471, buffer2, 987, 37);

                    if (size > 0)
                    {
                        incomingFile.Write(buffer2, 0, buffer2.Length);
                    }
                }
                incomingFile.Close();
                source.Close();
            }

            // cosmetic thing: set file creation, last write and last access to the original values
            File.SetCreationTime((adclFile.Substring(0, adclFile.Length - 1)) + "h", File.GetCreationTime(adclFile));
            File.SetLastWriteTime((adclFile.Substring(0, adclFile.Length - 1)) + "h", File.GetLastWriteTime(adclFile));
            File.SetLastAccessTime((adclFile.Substring(0, adclFile.Length - 1)) + "h", File.GetLastAccessTime(adclFile));

            // original .adcl file can be removed, copy was made on the earlier step
            if (File.Exists(Application.StartupPath + "\\old_Channels\\" + Path.GetFileName(adclFile)) && File.Exists(adclFile))
            {
                try
                {
                    File.Delete(adclFile);
                }
                catch
                {
                    MessageBox.Show("An error has occured: no rights to write to the current directory. Please fix this issue, you can try to copy this application and Channels directory to the other path and try again.", this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
            }
        }

        // launch URL to AltDVB website
        private void aboutLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start(aboutLink.Links[0].LinkData.ToString());
        }
    }
}
